import com.jenfa.*;
import com.motorola.io.FileConnection;
import java.io.*;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.io.*;
import motoftp.mftp;

public class SimpleFTP {

    Vector log;
    private SocketConnection socket;
    private AsciiReader reader;
    private AsciiWriter writer;
    FileConnection fc;
    private static boolean DEBUG = true;
    String SYSTEM;

    public SimpleFTP() {
        socket = null;
        reader = null;
        writer = null;
        fc = null;
        SYSTEM = "";
        log = new Vector();
    }

    public synchronized void connect(String host) throws IOException {
        connect(host, 21);
    }

    public synchronized void connect(String host, int port) throws IOException {
        connect(host, port, "anonymous", "anonymous");
    }

    public synchronized void connect(String host, int port, String user, String pass) throws IOException {
        if(socket != null)
            throw new IOException("MotoFTP is already connected. Disconnect first.");
        socket = (SocketConnection)Connector.open(String.valueOf(String.valueOf((new StringBuffer("socket://")).append(host).append(":").append(port))));
        try {
            reader = new AsciiReader();
            reader.openStream(socket.openInputStream());
            writer = new AsciiWriter();
            writer.openStream(socket.openOutputStream());
        }
        catch(Exception e) {
            Handle(e);
        }
        String response = readMultiLineAnswer();
        if(!response.startsWith("220 "))
            throw new IOException("MotoFTP received an unknown response when connecting to the FTP server: ".concat(String.valueOf(String.valueOf(response))));
        sendLine("USER ".concat(String.valueOf(String.valueOf(user))));
        response = readMultiLineAnswer();
        if(!response.startsWith("331 ") && !response.startsWith("230 "))
            throw new IOException("MotoFTP received an unknown response after sending the user: ".concat(String.valueOf(String.valueOf(response))));
        if(!response.startsWith("230 ")) {
            sendLine("PASS ".concat(String.valueOf(String.valueOf(pass))));
            response = readMultiLineAnswer();
            if(!response.startsWith("230 "))
                throw new IOException("MotoFTP was unable to log in with the supplied password: ".concat(String.valueOf(String.valueOf(response))));
        }
        sendLine("SYST");
        response = readMultiLineAnswer();
        if(!response.startsWith("215 ")) {
            if(DEBUG)
                System.out.println(String.valueOf(String.valueOf((new StringBuffer("MotoFTP cannot collect OS Info (")).append(response).append("). Falling back to UNIX"))));
            SYSTEM = "UNIX";
        } else {
            SYSTEM = response.substring(4);
        }
    }

    public synchronized void disconnect() throws IOException {
        sendLine("QUIT");
        socket.close();
        socket = null;
        break MISSING_BLOCK_LABEL_40;
        Exception exception;
        exception;
        socket.close();
        socket = null;
        throw exception;
    }

    public synchronized String pwd() throws IOException {
        sendLine("PWD");
        String dir = null;
        String response = readMultiLineAnswer();
        if(response.startsWith("257 ")) {
            int firstQuote = response.indexOf('"');
            int secondQuote = response.indexOf('"', firstQuote + 1);
            if(secondQuote > 0)
                dir = response.substring(firstQuote + 1, secondQuote);
        }
        return dir;
    }

    public synchronized boolean cwd(String dir) throws IOException {
        sendLine("CWD ".concat(String.valueOf(String.valueOf(dir))));
        String response = readMultiLineAnswer();
        return response.startsWith("250 ");
    }

    public String getLastURLPart(String URL, boolean hasFinalSlash) {
        int idx1 = 0;
        int idxf = -1;
        int pidxf = -1;
        idxf = URL.lastIndexOf('/');
        if(hasFinalSlash) {
            if(idxf != -1)
                pidxf = URL.lastIndexOf('/', idxf - 1);
            URL = URL.substring(pidxf + 1, idxf);
        } else {
            URL = URL.substring(idxf + 1);
        }
        return URL;
    }

    public synchronized boolean stor(String filename, mftp host, int timeout) throws IOException {
        String fnam = getLastURLPart(filename, false);
        fc = (FileConnection)Connector.open("file://".concat(String.valueOf(String.valueOf(filename))));
        return stor(fc.openInputStream(), fnam, host, timeout);
    }

    public synchronized Enumeration list() throws IOException {
        sendLine("PASV");
        String response = readMultiLineAnswer();
        if(!response.startsWith("227 "))
            throw new IOException("MotoFTP could not request passive mode: ".concat(String.valueOf(String.valueOf(response))));
        String ip = null;
        int port = -1;
        int opening = response.indexOf('(');
        int closing = response.indexOf(')', opening + 1);
        if(closing > 0) {
            String dataLink = response.substring(opening + 1, closing);
            StringTokenizer tokenizer = new StringTokenizer(dataLink, ",");
            try {
                ip = String.valueOf(String.valueOf((new StringBuffer(String.valueOf(String.valueOf(tokenizer.nextToken())))).append(".").append(tokenizer.nextToken()).append(".").append(tokenizer.nextToken()).append(".").append(tokenizer.nextToken())));
                port = Integer.parseInt(tokenizer.nextToken()) * 256 + Integer.parseInt(tokenizer.nextToken());
            }
            catch(Exception e) {
                throw new IOException("MotoFTP received bad data link information: ".concat(String.valueOf(String.valueOf(response))));
            }
        }
        sendLine("LIST");
        SocketConnection dSock = (SocketConnection)Connector.open(String.valueOf(String.valueOf((new StringBuffer("socket://")).append(ip).append(":").append(port))));
        response = readMultiLineAnswer();
        if(!response.startsWith("150 ") && !response.startsWith("125 "))
            throw new IOException("MotoFTP was not allowed to download filelist: ".concat(String.valueOf(String.valueOf(response))));
        AsciiReader list_read = new AsciiReader();
        Vector result = new Vector();
        UnixFileParser up = new UnixFileParser();
        WindowsFileParser wp = new WindowsFileParser();
        String line = "";
        byte parser = 1;
        if(SYSTEM.toUpperCase().startsWith("WINDOWS"))
            parser = 2;
        try {
            list_read.openStream(dSock.openInputStream());
            while(!list_read.eof())  {
                line = list_read.readLine();
                if(parser == 1)
                    result.addElement(up.parse(line));
                else
                    result.addElement(wp.parse(line));
            }
            list_read.close();
        }
        catch(Exception e) {
            if(e instanceof IOException)
                throw (IOException)e;
        }
        dSock.close();
        response = readMultiLineAnswer();
        Enumeration e1 = result.elements();
        return e1;
    }

    public synchronized boolean retr(String filename, mftp host, int timeout) throws Exception {
        String fnam = getLastURLPart(filename, false);
        try {
            fc = (FileConnection)Connector.open("file://".concat(String.valueOf(String.valueOf(filename))));
            if(fc.exists())
                fc.delete();
            fc.create();
        }
        catch(Exception e) {
            throw e;
        }
        return retr(fc.openOutputStream(), fnam, host, timeout);
    }

    public synchronized boolean retr(OutputStream output, String filename, mftp host, int timeout) throws IOException {
        sendLine("PASV");
        String response = readMultiLineAnswer();
        if(!response.startsWith("227 "))
            throw new IOException("MotoFTP could not request passive mode: ".concat(String.valueOf(String.valueOf(response))));
        String ip = null;
        int port = -1;
        int opening = response.indexOf('(');
        int closing = response.indexOf(')', opening + 1);
        if(closing > 0) {
            String dataLink = response.substring(opening + 1, closing);
            StringTokenizer tokenizer = new StringTokenizer(dataLink, ",");
            try {
                ip = String.valueOf(String.valueOf((new StringBuffer(String.valueOf(String.valueOf(tokenizer.nextToken())))).append(".").append(tokenizer.nextToken()).append(".").append(tokenizer.nextToken()).append(".").append(tokenizer.nextToken())));
                port = Integer.parseInt(tokenizer.nextToken()) * 256 + Integer.parseInt(tokenizer.nextToken());
            }
            catch(Exception e) {
                throw new IOException("MotoFTP received bad data link information: ".concat(String.valueOf(String.valueOf(response))));
            }
        }
        sendLine("RETR ".concat(String.valueOf(String.valueOf(filename))));
        SocketConnection dSock = (SocketConnection)Connector.open(String.valueOf(String.valueOf((new StringBuffer("socket://")).append(ip).append(":").append(port))));
        response = readMultiLineAnswer();
        if(!response.startsWith("150 ") && !response.startsWith("125 "))
            throw new IOException("MotoFTP was not allowed to download file: ".concat(String.valueOf(String.valueOf(response))));
        BufferedInputStream input = new BufferedInputStream(dSock.openInputStream());
        byte buffer[] = new byte[4096];
        for(int bytesRead = 0; (bytesRead = input.read(buffer)) != -1;) {
            output.write(buffer, 0, bytesRead);
            host.packet_downloaded(bytesRead);
        }

        output.flush();
        output.close();
        input.close();
        dSock.close();
        response = readMultiLineAnswer();
        return response.startsWith("226 ");
    }

    public synchronized boolean stor(InputStream inputStream, String filename, mftp host, int timeout) throws IOException {
        BufferedInputStream input = new BufferedInputStream(inputStream);
        sendLine("PASV");
        String response = readMultiLineAnswer();
        if(!response.startsWith("227 "))
            throw new IOException("MotoFTP could not request passive mode: ".concat(String.valueOf(String.valueOf(response))));
        String ip = null;
        int port = -1;
        int opening = response.indexOf('(');
        int closing = response.indexOf(')', opening + 1);
        if(closing > 0) {
            String dataLink = response.substring(opening + 1, closing);
            StringTokenizer tokenizer = new StringTokenizer(dataLink, ",");
            try {
                ip = String.valueOf(String.valueOf((new StringBuffer(String.valueOf(String.valueOf(tokenizer.nextToken())))).append(".").append(tokenizer.nextToken()).append(".").append(tokenizer.nextToken()).append(".").append(tokenizer.nextToken())));
                port = Integer.parseInt(tokenizer.nextToken()) * 256 + Integer.parseInt(tokenizer.nextToken());
            }
            catch(Exception e) {
                throw new IOException("MotoFTP received bad data link information: ".concat(String.valueOf(String.valueOf(response))));
            }
        }
        sendLine("STOR ".concat(String.valueOf(String.valueOf(filename))));
        SocketConnection dataSocket = (SocketConnection)Connector.open(String.valueOf(String.valueOf((new StringBuffer("socket://")).append(ip).append(":").append(port))));
        response = readMultiLineAnswer();
        if(!response.startsWith("150 ") && !response.startsWith("125 "))
            throw new IOException("MotoFTP was not allowed to send the file: ".concat(String.valueOf(String.valueOf(response))));
        BufferedOutputStream output = new BufferedOutputStream(dataSocket.openOutputStream());
        byte buffer[] = new byte[4096];
        for(int bytesRead = 0; (bytesRead = input.read(buffer)) != -1;) {
            output.write(buffer, 0, bytesRead);
            host.packet_uploaded(bytesRead);
        }

        output.flush();
        output.close();
        input.close();
        dataSocket.close();
        response = readMultiLineAnswer();
        return response.startsWith("226");
    }

    public synchronized boolean bin() throws IOException {
        sendLine("TYPE I");
        String response = readMultiLineAnswer();
        return response.startsWith("200 ");
    }

    public synchronized boolean mkd(String name) throws IOException {
        sendLine("MKD ".concat(String.valueOf(String.valueOf(name))));
        String response = readMultiLineAnswer();
        return response.startsWith("257 ");
    }

    public synchronized boolean rmd(String name) throws IOException {
        sendLine("RMD ".concat(String.valueOf(String.valueOf(name))));
        String response = readMultiLineAnswer();
        return response.startsWith("250 ");
    }

    public synchronized boolean dele(String name) throws IOException {
        sendLine("DELE ".concat(String.valueOf(String.valueOf(name))));
        String response = readMultiLineAnswer();
        return response.startsWith("250 ");
    }

    public synchronized boolean ascii() throws IOException {
        sendLine("TYPE A");
        String response = readMultiLineAnswer();
        return response.startsWith("200 ");
    }

    public Enumeration getLog() {
        return log.elements();
    }

    private void sendLine(String line) throws IOException {
        if(socket == null)
            throw new IOException("MotoFTP is not connected.");
        try {
            writer.writeLine(String.valueOf(String.valueOf(line)).concat("\r\n"));
            if(DEBUG)
                System.out.println("> ".concat(String.valueOf(String.valueOf(line))));
            log.addElement("> ".concat(String.valueOf(String.valueOf(line))));
        }
        catch(Exception e) {
            socket = null;
            throw (IOException)e;
        }
    }

    private String readMultiLineAnswer() throws IOException {
        String line = "";
        try {
            line = readLine();
        }
        catch(IOException ioe) {
            throw ioe;
        }
        while(line.charAt(3) == '-') 
            try {
                line = readLine();
            }
            catch(IOException ioe) {
                throw ioe;
            }
        return line;
    }

    private String readLine() throws IOException {
        try {
            String line = reader.readLine();
            if(DEBUG)
                System.out.println("< ".concat(String.valueOf(String.valueOf(line))));
            log.addElement("< ".concat(String.valueOf(String.valueOf(line))));
            String s = line;
            return s;
        }
        catch(Exception e) {
            Handle(e);
        }
        return "";
    }

    void Handle(Exception e) {
        System.out.println(e.toString());
    }

}
